from django.contrib.auth.models import AbstractUser, BaseUserManager
from django.db import models
import re
from django.core.exceptions import ValidationError
from datetime import date, datetime, timedelta
import sys 

# === VALIDASI ===
def validate_username(value):
    """Username hanya boleh mengandung huruf, angka, '.', '_', atau '-'."""
    if not re.match(r'^[a-zA-Z0-9._-]+$', value):
        raise ValidationError("Username hanya boleh berisi huruf, angka, '.', '_', atau '-'.")

def validate_password(value):
    """Password minimal 8 karakter dan harus mengandung karakter spesial."""
    if len(value) < 8 or not re.search(r'\W', value):
        raise ValidationError("Password harus minimal 8 karakter dan mengandung karakter spesial.")

def validate_birthdate(value):
    """Usia minimal adalah 12 tahun."""
    try:
        if isinstance(value, str):  # Jika masih dalam bentuk string
            value = datetime.strptime(value, "%d/%m/%Y").date()
        min_birthdate = date.today() - timedelta(days=12 * 365)
        if value > min_birthdate:
            raise ValidationError("Usia minimal adalah 12 tahun.")
        return value
    except ValueError:
        raise ValidationError("Format tanggal harus dd/mm/yyyy.")

def validate_phone(value):
    """Nomor HP harus terdiri dari 8-15 digit angka."""
    if not re.match(r'^[0-9]{8,15}$', value):
        raise ValidationError("Nomor HP harus terdiri dari 8-15 digit angka.")

def validate_nomor_rangka(value):
    """Nomor rangka harus diawali 5 huruf, lalu 10 karakter alfanumerik."""
    if not re.match(r'^[A-Za-z]{5}[A-Za-z0-9]{10}$', value):
        raise ValidationError("Nomor rangka harus diawali 5 huruf, lalu 10 karakter alfanumerik.")
    
def validate_blog_url(value):
    """Validasi URL harus diawali dengan http:// atau https://."""
    if not re.match(r'^https?://', value):
        raise ValidationError("URL harus diawali dengan 'http://' atau 'https://'.")
    return value

def validate_bio(value):
    """Bio harus antara 5 hingga 1000 karakter."""
    if not (5 <= len(value) <= 1000):
        raise ValidationError("Bio harus terdiri dari 5 hingga 1000 karakter.")
    return value

def validate_lokasi_duduk(value):
    """Lokasi duduk harus salah satu dari Jendela, Tengah, atau Gang."""
    pilihan_valid = ["Jendela", "Tengah", "Gang"]
    if value not in pilihan_valid:
        raise ValidationError(f"Lokasi duduk harus salah satu dari: {', '.join(pilihan_valid)}")
    return value

# === MODEL TRANSPORTASI ===
class Transportasi(models.Model):
    LOKASI_DUDUK_CHOICES = [
        ('Jendela', 'Jendela'),
        ('Tengah', 'Tengah'),
        ('Gang', 'Gang'),
    ]

    lokasi_duduk = models.CharField(
        max_length=10, 
        choices=LOKASI_DUDUK_CHOICES,
        verbose_name="Lokasi Duduk"
    )
    nomor_rangka = models.CharField(
        max_length=15, 
        unique=True, 
        validators=[validate_nomor_rangka],  
        verbose_name="Nomor Rangka Kendaraan"
    )

    def __str__(self):
        return f"{self.nomor_rangka} - {self.lokasi_duduk}"

# === CUSTOM USER MANAGER ===
class UserManager(BaseUserManager):
    def create_user(self, username, email, password, birthdate, phone_number, blog_url, bio, transportasi=None, **extra_fields):
        if not username:
            raise ValueError("Username harus diisi.")
        if not email:
            raise ValueError("Email harus diisi.")
        if not password:
            raise ValueError("Password harus diisi.")

        # Validasi
        validate_username(username)
        validate_password(password)
        birthdate = validate_birthdate(birthdate)
        validate_phone(phone_number)
        validate_blog_url(blog_url)
        validate_bio(bio)

        email = self.normalize_email(email)
        user = self.model(
            username=username,
            email=email,
            birthdate=birthdate,
            phone_number=phone_number,
            blog_url=blog_url,
            bio=bio,
            **extra_fields
        )
        user.set_password(password)
        if transportasi:
            user.transportasi = transportasi
        user.save(using=self._db)
        return user

    def create_superuser(self, username, email, password, **extra_fields):
        extra_fields.setdefault('is_staff', True)
        extra_fields.setdefault('is_superuser', True)

        errors = {}

        # Ambil input tambahan untuk superuser
        birthdate_str = input("Birthdate (dd/mm/yyyy): ")
        phone_number = input("Phone Number (8-15 angka): ")
        blog_url = input("Blog URL: ")
        bio = input("Bio (5-1000 karakter): ")
        lokasi_duduk = input("Lokasi Duduk (Jendela/Tengah/Gang): ")
        nomor_rangka = input("Nomor Rangka Kendaraan: ")

        # Validasi input
        try:
            birthdate = validate_birthdate(birthdate_str)
        except ValidationError as e:
            errors['Birthdate'] = str(e)

        try:
            validate_phone(phone_number)
        except ValidationError as e:
            errors['Phone Number'] = str(e)

        try:
            validate_blog_url(blog_url)
        except ValidationError as e:
            errors['Blog URL'] = str(e)

        try:
            validate_bio(bio)
        except ValidationError as e:
            errors['Bio'] = str(e)

        try:
            validate_lokasi_duduk(lokasi_duduk)
        except ValidationError as e:
            errors['Lokasi Duduk'] = str(e)

        try:
            validate_nomor_rangka(nomor_rangka)
        except ValidationError as e:
            errors['Nomor Rangka'] = str(e)

        # Jika ada error, tampilkan semua dan keluar
        if errors:
            print("\nERROR:")
            for field, error_msg in errors.items():
                if isinstance(error_msg, list):
                    error_msg = ", ".join(map(str, error_msg))  # Menghilangkan tanda kutip berlebih
                print(f" - {field}: {error_msg.strip('[]')}")
            sys.exit(1)

        # Buat objek transportasi terlebih dahulu
        transportasi = Transportasi.objects.create(
            lokasi_duduk=lokasi_duduk,
            nomor_rangka=nomor_rangka
        )

        return self.create_user(
            username=username,
            email=email,
            password=password,
            birthdate=birthdate,
            phone_number=phone_number,
            blog_url=blog_url,
            bio=bio,
            transportasi=transportasi,
            **extra_fields
        )

# === CUSTOM USER MODEL ===
class User(AbstractUser):
    username = models.CharField(max_length=255, unique=True, validators=[validate_username])
    password = models.CharField(max_length=255, validators=[validate_password])
    birthdate = models.DateField(validators=[validate_birthdate])
    phone_number = models.CharField(max_length=15, validators=[validate_phone])
    email = models.EmailField(unique=True)
    blog_url = models.URLField()
    bio = models.TextField(max_length=1000, blank=True)
    
    transportasi = models.OneToOneField(Transportasi, on_delete=models.SET_NULL, null=True, blank=True)
    
    objects = UserManager()
    
    def __str__(self):
        return self.username
